@@ -312,9 +312,10 @@ def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
         assert n == 2 or n == 3
 
         # Make pointer to the coordinate array
-        try:
+        if isinstance(array['data'], tuple):
+            # numpy tuple (addr, read-only)
             cp = cast(array['data'][0], POINTER(c_double))
-        except ArgumentError:
+        else:
             cp = array['data']
 
         # Add closing coordinates to sequence?
@@ -335,33 +336,21 @@ def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
 
         # add to coordinate sequence
         for i in xrange(m):
-            dx = c_double(cp[n*i])
-            dy = c_double(cp[n*i+1])
-            dz = None
-            if n == 3:
-                dz = c_double(cp[n*i+2])
-        
             # Because of a bug in the GEOS C API, 
             # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, i, dx)
-            lgeos.GEOSCoordSeq_setY(cs, i, dy)
+            lgeos.GEOSCoordSeq_setX(cs, i, cp[n*i])
+            lgeos.GEOSCoordSeq_setY(cs, i, cp[n*i+1])
             if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, i, dz)
+                lgeos.GEOSCoordSeq_setZ(cs, i, cp[n*i+2])
 
         # Add closing coordinates to sequence?
-        if M > m:
-            dx = c_double(cp[0])
-            dy = c_double(cp[1])
-            dz = None
-            if n == 3:
-                dz = c_double(cp[2])
-        
+        if M > m:        
             # Because of a bug in the GEOS C API, 
             # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, M-1, dx)
-            lgeos.GEOSCoordSeq_setY(cs, M-1, dy)
+            lgeos.GEOSCoordSeq_setX(cs, M-1, cp[0])
+            lgeos.GEOSCoordSeq_setY(cs, M-1, cp[1])
             if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, M-1, dz)
+                lgeos.GEOSCoordSeq_setZ(cs, M-1, cp[2])
             
     except AttributeError:
         # Fall back on list
@@ -391,34 +380,25 @@ def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
         # add to coordinate sequence
         for i in xrange(m):
             coords = ob[i]
-            dx = c_double(coords[0])
-            dy = c_double(coords[1])
-            dz = None
-            if n == 3:
-                dz = c_double(coords[2])
-        
             # Because of a bug in the GEOS C API, 
             # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, i, dx)
-            lgeos.GEOSCoordSeq_setY(cs, i, dy)
+            lgeos.GEOSCoordSeq_setX(cs, i, coords[0])
+            lgeos.GEOSCoordSeq_setY(cs, i, coords[1])
             if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, i, dz)
+                try:
+                    lgeos.GEOSCoordSeq_setZ(cs, i, coords[2])
+                except IndexError:
+                    raise ValueError("Inconsistent coordinate dimensionality")
 
         # Add closing coordinates to sequence?
         if M > m:
             coords = ob[0]
-            dx = c_double(coords[0])
-            dy = c_double(coords[1])
-            dz = None
-            if n == 3:
-                dz = c_double(coords[2])
-        
             # Because of a bug in the GEOS C API, 
             # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, M-1, dx)
-            lgeos.GEOSCoordSeq_setY(cs, M-1, dy)
+            lgeos.GEOSCoordSeq_setX(cs, M-1, coords[0])
+            lgeos.GEOSCoordSeq_setY(cs, M-1, coords[1])
             if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, M-1, dz)
+                lgeos.GEOSCoordSeq_setZ(cs, M-1, coords[2])
 
     if update_geom is not None:
         return None
